' *****************************************************************************
' Module:  BitArray.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
' *****************************************************************************


Imports System


'//////////////////////////////////////////////////////////////////////////////


Public Class BitArray
    A Private byte array that holds the bits
   Private byteArray() As Byte
   Private numBits As Int32
    This array avoids a time-consuming 2^n operation.
   Private bitMask() As Byte = {1, 2, 4, 8, 16, 32, 64, 128}

    Constructor that allocates the byte array and sets all bits to 0
   Public Sub New(ByVal numBits As Int32)
       Validate arguments first.
      If numBits <= 0 Then
         Throw New ArgumentOutOfRangeException(numBits must be > 0)
      End If

       Save the number of bits.
      Me.numBits = numBits
       Allocate the bytes for the bit array.
      ReDim byteArray((numBits - 1) \ 8)
   End Sub

    This is the parameterful property.
   Default Public Property Item(ByVal bitPos As Int32) As Boolean
       This is the propertys Get accessor method.
      Get
          Validate arguments first.
         If bitPos < 0 Or bitPos >= numBits Then
            Throw New IndexOutOfRangeException()
         End If
          Return the state of the indexed bit.
         Return CBool(byteArray(bitPos \ 8) And bitMask(bitPos Mod 8))
      End Get

       This is the propertys Set accessor method.
      Set(ByVal Value As Boolean)
         If bitPos < 0 Or bitPos >= numBits Then
            Throw New IndexOutOfRangeException()
         End If
         If Value Then
             Turn on the indexed bit.
            byteArray(bitPos \ 8) = byteArray(bitPos \ 8) Or _
               bitMask(bitPos Mod 8)
         Else
             Turn off the indexed bit.
            byteArray(bitPos \ 8) = byteArray(bitPos \ 8) And _
               Not bitMask(bitPos Mod 8)
         End If
      End Set
   End Property
End Class



' /////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
       Allocate a BitArray that can hold 14 bits.
      Dim ba As New BitArray(14)
       Turn on all the even-numbered bits by calling the Set accessor method.
      Dim x As Int32
      For x = 0 To 13
         ba(x) = ((x Mod 2) = 0)
      Next

       Show the state of all the bits by calling the Get accessor method.
      For x = 0 To 13
         Console.WriteLine (Bit {0} is {1}", x, ba(x))
      Next
   End Sub
End Class


' /////////////////////////////// End of File /////////////////////////////////
