'******************************************************************************
' Module:  MailManager.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
' *****************************************************************************


Imports System
Imports Microsoft.VisualBasic


' /////////////////////////////////////////////////////////////////////////////


Class MailManager
    The MailMsgEventArgs type is defined within the MailManager type.
   Public Class MailMsgEventArgs
      Inherits EventArgs

       1. Type defining information passed to receivers of the event
      Public Sub New(ByVal from As String, ByVal dest As String, _
         ByVal subject As String, ByVal body As String)
         Me.from = from
         Me.dest = dest
         Me.subject = subject
         Me.body = body
      End Sub

      Public ReadOnly from, dest, subject, body As String
   End Class

    2. Delegate type defining the prototype of the callback method
    that receivers must implement
   Public Delegate Sub MailMsgEventHandler(ByVal sender As Object, _
      ByVal args As MailMsgEventArgs)

    3. The event itself
   Public Event MailMsg As MailMsgEventHandler

    NOTE: Visual Basic lets you define the delegate implicitly by specifying
    a parameter signature in the Event declaration. The following
    (commented) event declaration is equivalent to steps 2 AND 3.
    Public Event MailMsg(ByVal sender As Object, _
    ByVal args As MailMsgEventArgs)

    4. Protected, virtual method responsible for notifying registered
    objects of the event
   Protected Overridable Sub OnMailMsg(ByVal e As MailMsgEventArgs)
       Notify all the objects in the delegate linked list.
      RaiseEvent MailMsg(Me, e)
   End Sub

    5. Method that translates the input into the desired event.
    This method is called when a new e-mail message arrives.
   Public Sub SimulateArrivingMsg(ByVal from As String, _
      ByVal dest As String, ByVal subject As String, ByVal body As String)
       Construct an object to hold the information you want
       to pass to the receivers of the notification.
      Dim e As New MailMsgEventArgs(from, dest, subject, body)

       Call the virtual method notifying the object that the event
       occurred. If no derived type overrides this method, the object
       will notify all the registered listeners.
      OnMailMsg(e)
   End Sub
End Class


' /////////////////////////////////////////////////////////////////////////////


Class Fax
    Pass the MailManager object to the constructor.
   Public Sub New(ByVal mm As MailManager)
       Construct an instance of the MailMsgEventHandler
       delegate that refers to the FaxMsg callback method.
       Register the callback method with MailManagers MailMsg event.
      AddHandler mm.MailMsg, _
         New MailManager.MailMsgEventHandler(AddressOf FaxMsg)
   End Sub

    This is the method that MailManager will call to
    notify the Fax object that a new e-mail message has arrived.
   Private Sub FaxMsg(ByVal sender As Object, _
      ByVal e As MailManager.MailMsgEventArgs)
       The sender identifies the MailManager in case
       you want to communicate back to it.

       The e identifies the additional event information
       that the MailManager wants to provide.

       Normally, the code here would fax the e-mail message. This test
       implementation displays the information on the console.
      Console.WriteLine(Faxing mail message:)
       Notice how you can embed carriage returns (CRs) and
       linefeeds (LFs) without using string concatenation.
      Console.WriteLine( From: {0}{4} To: {0}{4}  _
         & Subject: {2}{4} Body: {3}{4}", _
         e.from, e.dest, e.subject, e.body, ControlChars.CrLf)
   End Sub

   Public Sub Unregister(ByVal mm As MailManager)
       Construct an instance of the MailMsgEventHandler
       delegate that refers to the FaxMsg callback method.
      RemoveHandler mm.MailMsg, _
         New MailManager.MailMsgEventHandler(AddressOf FaxMsg)
   End Sub
End Class


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared WithEvents mm As MailManager

   Public Shared Sub Main()
       This statement registers the client so that it can receive events.
      mm = New MailManager()
       Let MailManager fire an event.
       (At this point, the MailMsgHandler procedure is invoked.)
      mm.SimulateArrivingMsg(Joe", Ann", Greetings", Hello, Ann)
       After the next statement, no more events can be received.
      mm = Nothing
   End Sub

   Shared Sub MailMsgHandler(ByVal sender As Object, _
      ByVal args As MailManager.MailMsgEventArgs) _
      Handles mm.MailMsg
      Console.WriteLine(The MailMsg event has been fired)
   End Sub
End Class


'/////////////////////////////// End of File //////////////////////////////////
