' *****************************************************************************
' Module:  DynamicDelegates.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
' *****************************************************************************


Imports System
Imports System.Reflection
Imports System.IO

 Here are some different delegate definitions.
Delegate Function TwoInt32s(ByVal n1 As Int32, ByVal n2 As Int32) As Object
Delegate Function OneString(ByVal s1 As String) As Object

' *****************************************************************************


Class App
   Shared Sub Main()
       Get arguments passed on the command line.
      Dim args() As String = Environment.GetCommandLineArgs

      If (args.Length < 3) Then
         Dim fileName As String = Path.GetFileNameWithoutExtension( _
         [Assembly].GetEntryAssembly().CodeBase)
         Console.WriteLine(Usage:)
         Console.WriteLine({0} delType methodName [Param1] [Param2]", _
            fileName)
         Console.WriteLine( where delType must be TwoInt32s or OneString)
         Console.WriteLine( if delType is TwoInt32s,  _
            & methodName must be Add or Subtract)
         Console.WriteLine( if delType is OneString,  _
            & methodName must be NumChars or Reverse)
         Console.WriteLine()
         Console.WriteLine(Examples:)
         Console.WriteLine( {0} TwoInt32s Add 123 321", fileName)
         Console.WriteLine( {0} TwoInt32s Subtract 123 321", fileName)
         Console.WriteLine( {0} OneString NumChars "Hello there""", _
            fileName)
         Console.WriteLine( {0} OneString Reverse "Hello there""", _
            fileName)
         Exit Sub
      End If

       args(0) is the name of the executable.
       args(1) is the delegate type.
      Dim delType As Type = Type.GetType(args(1))
      If delType Is Nothing Then
         Console.WriteLine(Invalid delType argument: {0}", args(1))
         Exit Sub
      End If

      Dim d As [Delegate]
      Try
          Create a delegate that points to the specific method.
          args(2) is the method name.
         d = [Delegate].CreateDelegate(delType, GetType(App), args(2))
      Catch ex As ArgumentException
         Console.WriteLine(Invalid methodName argument: {0}", args(2))
         Exit Sub
      End Try

       Prepare the array containing arguments.
      Dim callbackArgs(args.Length - 4) As Object

      If d.GetType() Is GetType(TwoInt32s) Then
          If the method takes two Int32s, retrieve
          them by parsing command arguments.
         Try
            Dim a As Int32
            For a = 3 To args.Length - 1
               callbackArgs(a - 3) = Int32.Parse(args(a))
            Next
         Catch ex As FormatException
            Console.WriteLine(Parameters must be integers.)
            Exit Sub
         End Try
      ElseIf d.GetType() Is GetType(OneString) Then
          If the method takes string arguments, copy them from the
          args array.
         Array.Copy(args, 3, callbackArgs, 0, callbackArgs.Length)
      End If

      Try
          Invoke the delegate dynamically and print the result.
         Dim result As Object = d.DynamicInvoke(callbackArgs)
         Console.WriteLine(Result = {0}", result)
      Catch ex As TargetParameterCountException
         Console.WriteLine(Incorrect number of parameters specified.)
      End Try
   End Sub

    This is the callback method that takes two Int32 parameters.
   Shared Function Add(ByVal n1 As Int32, ByVal n2 As Int32) As Object
      Return n1 + n2
   End Function

   Shared Function Subtract(ByVal n1 As Int32, ByVal n2 As Int32) As Object
      Return n1 - n2
   End Function

    This is the callback method that takes one String parameter.
   Shared Function NumChars(ByVal s1 As String) As Object
      Return s1.Length
   End Function

   Shared Function Reverse(ByVal s1 As String) As Object
      Dim chars() As Char = s1.ToCharArray()
      Array.Reverse(chars)
      Return New String(chars)
   End Function
End Class


'/////////////////////////////// End of File //////////////////////////////////
