' *****************************************************************************
' Module:  DiskFullException.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
' *****************************************************************************


Imports System
Imports System.IO
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters.Soap


' *****************************************************************************


 Allow instances of DiskFullException to be serialized.
<Serializable()> _
NotInheritable Class DiskFullException
   Inherits Exception

    The three public constructors
   Public Sub New()
      MyBase.New()  Call base constructor.
   End Sub

   Public Sub New(ByVal message As String)
      MyBase.New(message)  Call base constructor.
   End Sub

   Public Sub New(ByVal message As String, _
      ByVal innerException As Exception)
      MyBase.new(message, innerException)  Call base constructor.
   End Sub

    Define a private field.
   Private m_diskpath As String

    Define a read-only property that returns the field.
   ReadOnly Property DiskPath() As String
      Get
         Return m_diskpath
      End Get
   End Property

    Override the public Message property so that the
    field is included in the message.
   Public Overrides ReadOnly Property Message() As String
      Get
         Dim msg As String = MyBase.Message
         If Not m_diskpath Is Nothing Then
            msg &= Environment.NewLine & Disk Path:  + m_diskpath
         End If
         Return msg
      End Get
   End Property

    Because at least one field is defined, define the
    special deserialization constructor. Because this
    class is sealed, this constructor is private. If this
    class is not sealed, this constructor should be protected.
   Private Sub New (info as SerializationInfo, _
      context as StreamingContext )
       Let the base deserialize its fields.
      MyBase.new(info, context)
       Deserialize each field.
      m_diskpath = info.GetString(DiskPath)
   End Sub

    Because at least one field is defined,
    define the serialization method.
   Overrides Sub GetObjectData(ByVal info As SerializationInfo, _
      ByVal context As StreamingContext)
       Serialize each field.
      info.AddValue(DiskPath", m_diskpath)
       Let the base type serialize its fields.
      MyBase.GetObjectData(info, context)
   End Sub

    Define additional constructors that set the field.
   Public Sub New(ByVal message As String, ByVal diskpath As String)
      Me.New(message)  Call another constructor.
      m_diskpath = diskpath
   End Sub

   Public Sub New(ByVal message As String, ByVal diskpath As String, _
      ByVal innerException As Exception)
      Me.New(message, innerException)  Call another constructor.
      m_diskpath = diskpath
   End Sub
End Class


' *****************************************************************************


 The following code tests the serialization of the exception.
Class App
   Shared Sub Main()
       Construct a DiskFullException object, and serialize it.
      Dim e As New DiskFullException(The disk volume is full", C:\)
      Dim fs As New FileStream(Test", FileMode.Create)
      Dim f As IFormatter = New SoapFormatter()
      f.Serialize(fs, e)
      fs.Close()
       Deserialize the DiskFullException object, and check its fields.
      fs = New FileStream(Test", FileMode.Open)
      e = DirectCast(f.Deserialize(fs), DiskFullException)
      fs.Close()
      Console.WriteLine(Type: {1}{0}DiskPath: {2}{0}Message: {3}", _
      Environment.NewLine, e.GetType(), e.DiskPath, e.Message)
   End Sub
End Class


' //////////////////////////////// End of File ////////////////////////////////
