'******************************************************************************
' Module:  ExceptionTree.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
'******************************************************************************


Imports System
Imports System.Text
Imports System.Reflection
Imports System.Collections


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
      ' Explicitly load the assemblies that I want to reflect over.
      LoadAssemblies()

      ' Initialize the counters and the exception type list.
      Dim totalTypes, totalExceptionTypes As Int32
      Dim exceptionTree As New ArrayList()

      ' Iterate through all assemblies loaded in this AppDomain.
      Dim a As [Assembly]
      For Each a In AppDomain.CurrentDomain.GetAssemblies
         ' Iterate through all types defined in this assembly.
         Dim t As Type
         For Each t In a.GetTypes
            totalTypes += 1
            ' Ignore type if not a public class.
            If t.IsClass Or t.IsPublic Then
               ' Build a string of the types derivation hierarchy.
               Dim typeHierarchy As New StringBuilder(t.FullName, 5000)
               ' Assume that the type isnt an Exception-derived type.
               Dim derivedFromException As Boolean = False
               ' See if System.Exception is a base type of this type.
               Dim baseType As Type = t.BaseType
               Do Until baseType Is Nothing Or derivedFromException
                  ' Append the base type to the end of the string.
                  typeHierarchy.Append(-)
                  typeHierarchy.Append(baseType)
                  derivedFromException = baseType Is GetType(Exception)
                  baseType = baseType.BaseType
               Loop
               ' No more bases and not Exception-derived, so try next type.
               If derivedFromException Then
                  ' I found an Exception-derived type.
                  totalExceptionTypes += 1
                  ' For this Exception-derived type, reverse the order
                  ' of the types in the hierarchy.
                  Dim h() As String = typeHierarchy.ToString().Split(-"c)
                  Array.Reverse(h)
                  ' Build a new string with the hierarchy in order
                  ' from Exception -> Exception-derived type.
                  ' Add the string to the list of Exception types.
                  exceptionTree.Add(String.Join(-", h, 1, h.Length - 1))
               End If
            End If
         Next
         ' Sort the Exception types together in order of their hierarchy.
         exceptionTree.Sort()
         ' Display the Exception tree.
         Dim s As String
         For Each s In exceptionTree
            ' For this Exception type, split its base types apart.
            Dim x() As String = s.Split(-"c)
            ' Indent based on the number of base types,
            ' and then show the most derived type.
            Console.WriteLine(New String( c, 3 * x.Length) + _
               x(x.Length - 1))
         Next
      Next
      ' Show the final status of the types considered.
      Console.WriteLine()
      Console.WriteLine(---> Of {0} types, {1} are  & _
         derived from System.Exception.", totalTypes, totalExceptionTypes)
      Console.ReadLine()
   End Sub

   Shared Sub LoadAssemblies()
      Dim assemblies() As String = { _
         System, PublicKeyToken={0}", _
         System.Data, PublicKeyToken={0}", _
         System.Design, PublicKeyToken={1}", _
         System.DirectoryServices, PublicKeyToken={1}", _
         System.Drawing, PublicKeyToken={1}", _
         System.Drawing.Design, PublicKeyToken={1}", _
         System.EnterpriseServices, PublicKeyToken={1}", _
         System.Management, PublicKeyToken={1}", _
         System.Messaging, PublicKeyToken={1}", _
         System.Runtime.Remoting, PublicKeyToken={0}", _
         System.Security, PublicKeyToken={1}", _
         System.ServiceProcess, PublicKeyToken={1}", _
         System.Web, PublicKeyToken={1}", _
         System.Web.RegularExpressions, PublicKeyToken={1}", _
         System.Web.Services, PublicKeyToken={1}", _
         System.Windows.Forms, PublicKeyToken={0}", _
         System.Xml, PublicKeyToken={0} _
      }
      Dim EcmaPublicKeyToken As String = b77a5c561934e089"
      Dim MSPublicKeyToken As String = b03f5f7f11d50a3a"
      ' Get the version of the assembly containing System.Object.
      ' Ill assume the same version for all the other assemblies.
      Dim version As Version = _
         GetType(System.Object).Assembly.GetName().Version()
      ' Explicitly load the assemblies that I want to reflect over.
      Dim a As String
      For Each a In assemblies
         Dim AssemblyIdentity As String = String.Format(a, _
            EcmaPublicKeyToken, MSPublicKeyToken) & _
            , Culture=neutral, Version= & version.ToString
            [Assembly].Load(AssemblyIdentity)
      Next
   End Sub
End Class


'/////////////////////////////// End of File //////////////////////////////////
