'******************************************************************************
' Module:  FindInterfaces.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
'******************************************************************************


Imports System
Imports System.Reflection


'//////////////////////////////////////////////////////////////////////////////


' Define two interfaces for testing.
Public Interface IBookRetailer
   Inherits IDisposable
   Sub Purchase()
   Sub ApplyDiscount()
End Interface


'//////////////////////////////////////////////////////////////////////////////


Public Interface IMusicRetailer
   Sub Purchase()
End Interface


'//////////////////////////////////////////////////////////////////////////////


' This class implements two interfaces defined by this
' assembly and one interface defined by another assembly.
Class MyRetailer
   Implements IBookRetailer, IMusicRetailer
   Public Sub Purchase()
   End Sub

   Public Sub Dispose() Implements IDisposable.Dispose
   End Sub

   Sub IBookRetailer_Purchase() Implements IBookRetailer.Purchase
   End Sub

   Public Sub ApplyDiscount() Implements IBookRetailer.ApplyDiscount
   End Sub

   Sub IMusicRetailer_Purchase() Implements IMusicRetailer.Purchase
   End Sub
End Class


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
      ' Find the interfaces implemented by MyRetailer where
      ' the interface is defined in your own assembly. This
      ' is accomplished using a delegate to a filter method
      ' that you create and pass to FindInterfaces.
      Dim t As Type = GetType(MyRetailer)
      Dim interfaces() As Type = t.FindInterfaces( _
         New TypeFilter(AddressOf App.TypeFilter), _
         [Assembly].GetCallingAssembly().GetName())
      Console.WriteLine(MyRetailer implements the following  _
         & interfaces (defined in this assembly):)

      ' Show information about each interface.
      Dim i As Type
      For Each i In interfaces
         Console.WriteLine()
         Console.WriteLine(Interface: {0}", i)

         ' Get the type methods that map to the interfaces methods.
         Dim map As InterfaceMapping = t.GetInterfaceMap(i)
         Dim m As Int32
         For m = 0 To map.InterfaceMethods.Length - 1
            ' Display the interface method name and which type
            ' method implements the interface method.
            Console.WriteLine( {0} is implemented by {1}", _
               map.InterfaceMethods(m), map.TargetMethods(m))
         Next
      Next
   End Sub

   ' This filter delegate method takes a type and an object, performs
   ' some check, and returns True if the type is to be included
   ' in the array of returned types.
   Shared Function TypeFilter(ByVal t As Type, _
      ByVal filterCriteria As Object) As Boolean
      ' Return True if the interface is defined in the same
      ' assembly identified by filterCriteria.
      Return t.Assembly.GetName.ToString() = filterCriteria.ToString()
   End Function
End Class


'/////////////////////////////// End of File //////////////////////////////////
