'******************************************************************************
' Module:  Invoker.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
'******************************************************************************


' Modify the following line to test,
' bind, and invoke as separate steps.
#Const BindAndInvokeTogether = False


'//////////////////////////////////////////////////////////////////////////////


Imports System
Imports System.Reflection
Imports System.Threading


'//////////////////////////////////////////////////////////////////////////////


' This class is used to demonstrate reflection. It has a
' a field, a constructor, a method, a property, and an event.
Class SomeType
   Dim someField As Int32
   Public Sub New(ByRef x As Int32)
      x *= 2
   End Sub

   Public Overrides Function ToString() As String
      Return someField.ToString
   End Function

   Property SomeProp() As Int32
      Get
         Return someField
      End Get
      Set(ByVal Value As Int32)
         If Value < 1 Then
            Throw New ArgumentOutOfRangeException(value", value, _
               value must be > 0)
         End If
         someField = Value
      End Set
   End Property

   Public Event SomeEvent As ThreadStart
End Class


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
      Dim t As Type = GetType(SomeType)

      Dim bf As BindingFlags = BindingFlags.DeclaredOnly Or _
         BindingFlags.Public Or BindingFlags.NonPublic Or _
         BindingFlags.Instance
   
#If BindAndInvokeTogether Then

      ' Construct an instance of the type.
      Dim args() As Object = {12}  Constructor arguments
      Console.WriteLine(x before constructor called: {0}", args(0))
      Dim obj As Object = t.InvokeMember(Nothing, _
         bf Or BindingFlags.CreateInstance, Nothing, Nothing, args)
      Console.WriteLine(Type: {0}", obj.GetType().ToString)
      Console.WriteLine(x after constructor returns: {0}", args(0))

      ' Read from and write to a field.
      t.InvokeMember(someField", bf Or BindingFlags.SetField, _
         Nothing, obj, New Object() {5})
      Dim v As Int32 = CType(t.InvokeMember(someField", _
         bf Or BindingFlags.GetField, Nothing, obj, Nothing), Int32)
      Console.WriteLine(someField: {0}", v)

      ' Call a method.
      Dim s As String = DirectCast(t.InvokeMember(ToString", _
         bf Or BindingFlags.InvokeMethod, Nothing, obj, Nothing), String)
      Console.WriteLine(ToString: {0}", s)

      ' Read from and write to a property.
      Try
         t.InvokeMember(SomeProp", bf Or BindingFlags.SetProperty, _
            Nothing, obj, New Object() {0})
      Catch e As TargetInvocationException
         If Not e.InnerException.GetType Is _
            GetType(ArgumentOutOfRangeException) Then Throw
            Console.WriteLine(Property set catch.)
      End Try

      t.InvokeMember(SomeProp", bf Or BindingFlags.SetProperty, _
         Nothing, obj, New Object() {2})
      v = CType(t.InvokeMember(SomeProp", _
         bf Or BindingFlags.GetProperty, Nothing, obj, Nothing), Int32)
      Console.WriteLine(SomeProp: {0}", v)

      ' NOTE: InvokeMember doesnt support events.

#Else

      ' Construct an instance.
      Dim ctor as ConstructorInfo = t.GetConstructor(New Type() _
         { Type.GetType(System.Int32&) })
      Dim args() As Object = { 12 }  Constructor arguments
      Console.WriteLine(x before constructor called: {0} , args(0))
      Dim obj As Object = ctor.Invoke(args)
      Console.WriteLine(Type: {0} , obj.GetType().ToString)
      Console.WriteLine(x after constructor returns: {0} , args(0))

      ' Read from and write to a field.
      Dim fi as FieldInfo = obj.GetType().GetField(someField", bf)
      fi.SetValue(obj, 33)
      Console.WriteLine(someField: {0} , fi.GetValue(obj))

      ' Call a method.
      Dim mi as MethodInfo= obj.GetType().GetMethod(ToString", bf)
      Dim s as String= DirectCast(mi.Invoke(obj, Nothing), String)
      Console.WriteLine(ToString: {0} , s)

      ' Read from and write to a property.
      Dim pi As PropertyInfo = obj.GetType().GetProperty( _
         SomeProp", GetType(Int32))
      Dim m as MethodInfo
      For each m in pi.GetAccessors
         Console.WriteLine(m)
      Next
      Try
         pi.SetValue(obj, 0, Nothing)
      Catch e As TargetInvocationException
         If Not e.InnerException.GetType() Is _
            GetType(ArgumentOutOfRangeException) Then Throw
         Console.WriteLine(Property set catch.)
      End Try

      pi.SetValue(obj, 2, Nothing)
      Console.WriteLine(SomeProp: {0}", pi.GetValue(obj, Nothing))

      ' Add and remove a delegate from the event.
      Dim ei As EventInfo = obj.GetType().GetEvent(SomeEvent", bf)
      Console.WriteLine(AddMethod: {0}", ei.GetAddMethod)
      Console.WriteLine(RemoveMethod: {0}", ei.GetRemoveMethod)
      Console.WriteLine(EventHandlerType: {0}", ei.EventHandlerType)

      Dim ts As New ThreadStart(AddressOf Main)
      ei.AddEventHandler(obj, ts)
      ei.RemoveEventHandler(obj, ts)

#End If

   End Sub
End Class


'/////////////////////////////// End of File //////////////////////////////////
