'******************************************************************************
' Module:  Reflector.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
'******************************************************************************


' Define Ver as either 1, 2, or 3 to match the example from the book
#Const Ver = 3


'//////////////////////////////////////////////////////////////////////////////

Imports System
Imports System.Reflection


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
#If Ver = 1 Or Ver = 3 Then
      ' Notice that Assembly is a Visual Basic keyword.
      Dim assem As [Assembly] = [Assembly].GetExecutingAssembly()
      Reflector.ReflectOnAssembly(assem)
#End If
#If Ver = 2 Then
      Dim assem As [Assembly]
      For Each assem In AppDomain.CurrentDomain.GetAssemblies()
         Reflector.ReflectOnAssembly(assem)
      Next
#End If
      Console.WriteLine("Press <Enter> to exit.")
      Console.ReadLine()
   End Sub
End Class


'//////////////////////////////////////////////////////////////////////////////


Public Class Reflector
   Public Shared Sub ReflectOnAssembly(ByVal assem As [Assembly])
      WriteLine(0, Assembly: {0}", assem)
      ' Find modules.
      ' Notice that Module is a Visual Basic keyword.
      Dim m As [Module]
      For Each m In assem.GetModules
         WriteLine(1, Module: {0}", m)

         ' Find types.
         Dim t As Type
         For Each t In m.GetTypes
            WriteLine(2, Type: {0}", t)

            ' Find members.
#If Ver = 1 Or Ver = 2 Then
            Dim mi As MemberInfo
            For Each mi In t.GetMembers
               WriteLine(3, {0}: {1}", mi.MemberType, mi)
            Next
#End If
#If Ver = 3 Then
            Dim bf as BindingFlags = BindingFlags.DeclaredOnly Or _ 
               BindingFlags.NonPublic Or BindingFlags.Public Or _
               BindingFlags.Instance  Or BindingFlags.Static

            Dim mi As MemberInfo
            For Each mi in t.GetMembers(bf)
               WriteLine(3, "{0}: {1}", mi.MemberType, mi)
            Next
#End If
         Next
      Next
   End Sub

   Private Shared Sub WriteLine(ByVal indent As Int32, _
      ByVal format As String, ByVal ParamArray args() As Object)
      Console.WriteLine(New String( c, 3 * indent) & format, args)
   End Sub
End Class


'//////////////////////////////////////////////////////////////////////////////


Class SomeType
   Public Class InnerType
   End Class

   Public SomeField As Int32
   Private Shared goo As String

   Private Sub SomeMethod()
   End Sub

   Private Property SomeProperty() As TimeSpan
      Get
         Return New TimeSpan()
      End Get
      Set(ByVal Value As TimeSpan)
      End Set
   End Property
   Public Shared Event SomeEvent As System.Threading.ThreadStart
End Class


'/////////////////////////////// End of File //////////////////////////////////
