'******************************************************************************
' Module:  SimpleBinderTester.vb
' Notices: Copyright (c) 2002 Jeffrey Richter
' Thanks:  To Dario Russi for supplying the initial version of this code.
'******************************************************************************


Option Strict On
Option Explicit On 


Imports System
Imports System.Reflection
Imports System.Diagnostics


'//////////////////////////////////////////////////////////////////////////////


Class App
   Shared Sub Main()
      Dim o As Object = New SomeType()
      Dim t As Type = GetType(SomeType)
      Dim myBinder As Binder = New SimpleBinder()
      Dim bf As BindingFlags = BindingFlags.Public Or BindingFlags.Instance

      Dim b As Int16 = 5

      ' Calls Void v(Int32)
      t.InvokeMember("v", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {b})

      ' Calls Void m(Int32)
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {1})

      ' Calls Void m(System.Object)
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {New Object()})

      ' Calls Void m(Double)
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {1.4})

      ' Calls Void m(SomeType)
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {o})

      ' Calls Void m(System.Object) since m(String) is private
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {"string"})

      ' Calls Void m(System.String) since NonPublic is specified
      t.InvokeMember("m", bf Or BindingFlags.NonPublic Or BindingFlags.InvokeMethod, myBinder, o, New Object() {"string"})

      Try
         ' Throws because there is no public method which takes exactly a string 
         t.InvokeMember("m", bf Or BindingFlags.InvokeMethod Or BindingFlags.ExactBinding, myBinder, o, New Object() {"string"})
      Catch ex As MissingMethodException
         Console.WriteLine("Invocation failed on m(String), bad binding flags - ExactBinding too restrictive")
      End Try

      Try
         ' Throws because there is no method g which takes only 2 args 
         t.InvokeMember("g", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {1, "string"})
      Catch ex As MissingMethodException
         Console.WriteLine("Invocation failed on g(int, Object, String), wrong number of args")
      End Try

      ' Calls Void g(Int32, System.Object, System.String)
      t.InvokeMember("g", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {1, "string", "string"})

      ' Calls Void h()
      t.InvokeMember("h", bf Or BindingFlags.NonPublic Or BindingFlags.InvokeMethod, myBinder, o, New Object() {})

      Try
         ' Throws because BindingFlags.Static has not been specified 
         t.InvokeMember("s", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {})
      Catch ex As MissingMethodException
         Console.WriteLine("Invocation failed on static s(), bad binding flags - need Static")
      End Try

      ' Calls Void s()
      t.InvokeMember("s", bf Or BindingFlags.InvokeMethod Or BindingFlags.Static, myBinder, o, New Object() {})

      ' Calls Void m(Int32, Double)
      t.InvokeMember("m", bf Or BindingFlags.InvokeMethod, myBinder, o, New Object() {1, 1})

      Console.WriteLine("Press <Enter> to exit.")
      Console.ReadLine()
   End Sub
End Class


'//////////////////////////////////////////////////////////////////////////////


' A simple class with a bunch of members to test the Binder
Class SomeType
   Public Sub m(ByVal i As Int32)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub m(ByVal d As Double)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub m(ByVal o As Object)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub m(ByVal s As SomeType)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub m(ByVal i As Int32, ByVal m As Double)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Private Sub m(ByVal s As String)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub g(ByVal i As Int32, ByVal o As Object, ByVal s As String)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Private Sub h()
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Shared Sub s()
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub

   Public Sub v(ByVal s As Int32)
      Console.WriteLine(New StackTrace().GetFrame(0).GetMethod().ToString())
   End Sub
End Class


'//////////////////////////////// End of File /////////////////////////////////
